@@ -1,6 +1,8 @@
 import setuptools  # noqa
 from setuptools import Extension
 import sys
+from setuptools.command.build_ext import (  # noqa: N812
+        build_ext as BaseBuildExtCommand)
 
 
 def count_down_delay(delay):
@@ -12,6 +14,7 @@ def count_down_delay(delay):
         sleep(1)
     print("")
 
+
 DASH_SEPARATOR = 75 * "-"
 
 
@@ -23,7 +26,7 @@ def setup(*args, **kwargs):
         raise
     except SystemExit:
         raise
-    except:
+    except Exception:
         print(DASH_SEPARATOR)
         print("Sorry, your build failed. Try rerunning configure.py with "
                 "different options.")
@@ -31,6 +34,14 @@ def setup(*args, **kwargs):
         raise
 
 
+def get_numpy_incpath():
+    from imp import find_module
+    # avoid actually importing numpy, it screws up distutils
+    file, pathname, descr = find_module("numpy")
+    from os.path import join
+    return join(pathname, "core", "include")
+
+
 class NumpyExtension(Extension):
     # nicked from
     # http://mail.python.org/pipermail/distutils-sig/2007-September/008253.html
@@ -40,15 +51,8 @@ class NumpyExtension(Extension):
         self._include_dirs = self.include_dirs
         del self.include_dirs  # restore overwritten property
 
-    def get_numpy_incpath(self):
-        from imp import find_module
-        # avoid actually importing numpy, it screws up distutils
-        file, pathname, descr = find_module("numpy")
-        from os.path import join
-        return join(pathname, "core", "include")
-
     def get_additional_include_dirs(self):
-        return [self.get_numpy_incpath()]
+        return [get_numpy_incpath()]
 
     def get_include_dirs(self):
         return self._include_dirs + self.get_additional_include_dirs()
@@ -62,35 +66,44 @@ class NumpyExtension(Extension):
     include_dirs = property(get_include_dirs, set_include_dirs, del_include_dirs)
 
 
-class PyUblasExtension(NumpyExtension):
-    def get_module_include_path(self, name):
-        from pkg_resources import Requirement, resource_filename
-        return resource_filename(Requirement.parse(name), "%s/include" % name)
+class ExtensionUsingNumpy(Extension):
+    """Unlike :class:`NumpyExtension`, this class does not require numpy to be
+    importable upon extension module creation, allowing ``setup_requires=["numpy"]``
+    to work. On the other hand, it requires the use of::
 
-    def get_additional_include_dirs(self):
-        return (NumpyExtension.get_additional_include_dirs(self)
-                + [self.get_module_include_path("pyublas")])
+        setup(...,
+            cmdclass={'build_ext': NumpyBuildExtCommand})
+
+    or
+
+        setup(...,
+            cmdclass={'build_ext': PybindBuildExtCommand})
+    """
 
 
-class HedgeExtension(PyUblasExtension):
-    @property
-    def include_dirs(self):
-        return self._include_dirs + [
-                self.get_numpy_incpath(),
-                self.get_module_include_path("pyublas"),
-                self.get_module_include_path("hedge"),
-                ]
+class NumpyBuildExtCommand(BaseBuildExtCommand):
+    def build_extension(self, extension):
+        # We add the numpy include dir right before building the
+        # extension, in order to avoid having to import numpy when
+        # the setup script is imported, which would prevent
+        # installation before manual installation of numpy.
+        if isinstance(extension, ExtensionUsingNumpy):
+            numpy_incpath = get_numpy_incpath()
+            if numpy_incpath not in extension.include_dirs:
+                extension.include_dirs.append(numpy_incpath)
+
+        BaseBuildExtCommand.build_extension(self, extension)
 
 
 # {{{ tools
 
-def flatten(list):
+def flatten(lst):
     """For an iterable of sub-iterables, generate each member of each
     sub-iterable in turn, i.e. a flattened version of that super-iterable.
 
     Example: Turn [[a,b,c],[d,e,f]] into [a,b,c,d,e,f].
     """
-    for sublist in list:
+    for sublist in lst:
         for j in sublist:
             yield j
 
@@ -149,10 +162,12 @@ def hack_distutils(debug=False, fast_link=True, what_opt=3):
         from distutils import sysconfig
 
         cvars = sysconfig.get_config_vars()
-        cflags = cvars.get('OPT')
+
+        bad_prefixes = ['-g', '-O', '-Wstrict-prototypes', '-DNDEBUG']
+
+        cflags = cvars.get("OPT")
         if cflags:
-            cflags = remove_prefixes(cflags.split(),
-                    ['-g', '-O', '-Wstrict-prototypes', '-DNDEBUG'])
+            cflags = remove_prefixes(cflags.split(), bad_prefixes)
             if debug:
                 cflags.append("-g")
             else:
@@ -162,11 +177,17 @@ def hack_distutils(debug=False, fast_link=True, what_opt=3):
                     cflags.append("-O%s" % what_opt)
                     cflags.append("-DNDEBUG")
 
-            cvars['OPT'] = str.join(' ', cflags)
-            if "BASECFLAGS" in cvars:
-                cvars["CFLAGS"] = cvars["BASECFLAGS"] + " " + cvars["OPT"]
-            else:
-                assert "CFLAGS" in cvars
+            cvars["OPT"] = str.join(' ', cflags)
+
+        cflags = cvars.get("CONFIGURE_CFLAGS")
+        if cflags:
+            cflags = remove_prefixes(cflags.split(), bad_prefixes)
+            cvars["CONFIGURE_CFLAGS"] = str.join(' ', cflags)
+
+        if "BASECFLAGS" in cvars:
+            cvars["CFLAGS"] = cvars["BASECFLAGS"] + " " + cvars.get("OPT", "")
+        else:
+            assert "CFLAGS" in cvars
 
         if fast_link:
             for varname in ["LDSHARED", "BLDSHARED"]:
@@ -212,7 +233,7 @@ def expand_value(v, options):
         for i in v:
             try:
                 exp_i = expand_value(i, options)
-            except:
+            except Exception:
                 pass
             else:
                 result.append(exp_i)
@@ -503,6 +524,10 @@ class Libraries(StringListOption):
                 help=help or ("Library names for %s (without lib or .so)"
                 % (human_name or humanize(lib_name))))
 
+# }}}
+
+
+# {{{ configure options for specific software
 
 class BoostLibraries(Libraries):
     def __init__(self, lib_base_name, default_lib_name=None):
@@ -627,6 +652,10 @@ def make_boost_base_options():
             help="The compiler with which Boost C++ was compiled, e.g. gcc43"),
         ]
 
+# }}}
+
+
+# {{{ configure frontend
 
 def configure_frontend():
     from optparse import OptionParser
@@ -678,6 +707,8 @@ def configure_frontend():
 
         substitute(substs, "Makefile")
 
+# }}}
+
 
 def substitute(substitutions, fname):
     import re
@@ -685,19 +716,24 @@ def substitute(substitutions, fname):
     string_var_re = re.compile(r"\$str\{([A-Za-z_0-9]+)\}")
 
     fname_in = fname+".in"
-    lines = open(fname_in, "r").readlines()
+    with open(fname_in, "r") as inf:
+        lines = inf.readlines()
+
     new_lines = []
-    for l in lines:
+    for line in lines:
         made_change = True
         while made_change:
             made_change = False
-            match = var_re.search(l)
+            match = var_re.search(line)
             if match:
                 varname = match.group(1)
-                l = l[:match.start()] + str(substitutions[varname]) + l[match.end():]
+                line = (
+                        line[:match.start()]
+                        + str(substitutions[varname])
+                        + line[match.end():])
                 made_change = True
 
-            match = string_var_re.search(l)
+            match = string_var_re.search(line)
             if match:
                 varname = match.group(1)
                 subst = substitutions[varname]
@@ -706,19 +742,22 @@ def substitute(substitutions, fname):
                 else:
                     subst = '"%s"' % subst
 
-                l = l[:match.start()] + subst + l[match.end():]
+                line = line[:match.start()] + subst + line[match.end():]
                 made_change = True
-        new_lines.append(l)
+        new_lines.append(line)
     new_lines.insert(1, "# DO NOT EDIT THIS FILE -- "
             "it was generated by configure.py\n")
     new_lines.insert(2, "# %s\n" % (" ".join(sys.argv)))
-    open(fname, "w").write("".join(new_lines))
+    with open(fname, "w") as outf:
+        outf.write("".join(new_lines))
 
     from os import stat, chmod
     infile_stat_res = stat(fname_in)
     chmod(fname, infile_stat_res.st_mode)
 
 
+# {{{ git bits
+
 def _run_git_command(cmd):
     git_error = None
     from subprocess import Popen, PIPE
@@ -823,3 +862,129 @@ def check_git_submodules():
                     print(DASH_SEPARATOR)
                     print("git submodules initialized successfully")
                     print(DASH_SEPARATOR)
+
+# }}}
+
+
+# {{{ pybind11
+
+def check_pybind11():
+    try:
+        import pybind11  # noqa
+    except ImportError:
+        print(DASH_SEPARATOR)
+        print("Pybind11 is not installed.")
+        print(DASH_SEPARATOR)
+        print("Very likely, the build process after this message will fail.")
+        print("")
+        print("Simply press Ctrl+C and type")
+        print("python -m pip install pybind11")
+        print("to fix this. If you don't, the build will continue ")
+        print("in a few seconds.")
+        print("")
+        print("[1] https://pybind11.readthedocs.io/en/stable/")
+        print(DASH_SEPARATOR)
+
+        from aksetup_helper import count_down_delay
+        count_down_delay(delay=10)
+
+
+# {{{ (modified) boilerplate from https://github.com/pybind/python_example/blob/2ed5a68759cd6ff5d2e5992a91f08616ef457b5c/setup.py  # noqa
+
+class get_pybind_include(object):  # noqa: N801
+    """Helper class to determine the pybind11 include path
+
+    The purpose of this class is to postpone importing pybind11
+    until it is actually installed, so that the ``get_include()``
+    method can be invoked. """
+
+    def __init__(self, user=False):
+        self.user = user
+
+    def __str__(self):
+        import pybind11
+        return pybind11.get_include(self.user)
+
+
+# As of Python 3.6, CCompiler has a `has_flag` method.
+# cf http://bugs.python.org/issue26689
+def has_flag(compiler, flagname):
+    """Return a boolean indicating whether a flag name is supported on
+    the specified compiler.
+    """
+    import tempfile
+    with tempfile.NamedTemporaryFile('w', suffix='.cpp', delete=False) as f:
+        f.write('int main (int argc, char **argv) { return 0; }')
+        fname = f.name
+    try:
+        compiler.compile([fname], extra_postargs=[flagname])
+    except setuptools.distutils.errors.CompileError:
+        return False
+    return True
+
+
+def cpp_flag(compiler):
+    """Return the -std=c++[11/14] compiler flag.
+
+    The c++14 is prefered over c++11 (when it is available).
+    """
+    if has_flag(compiler, '-std=gnu++14'):
+        return '-std=gnu++14'
+    elif has_flag(compiler, '-std=c++14'):
+        return '-std=c++14'
+    elif has_flag(compiler, '-std=c++11'):
+        return '-std=c++11'
+    else:
+        raise RuntimeError('Unsupported compiler -- at least C++11 support '
+                           'is needed!')
+
+
+class PybindBuildExtCommand(NumpyBuildExtCommand):
+    """A custom build extension for adding compiler-specific options."""
+    c_opts = {
+        'msvc': ['/EHsc'],
+        'unix': [],
+    }
+
+    def build_extensions(self):
+        ct = self.compiler.compiler_type
+        opts = self.c_opts.get(ct, [])
+        cxx_opts = []
+
+        if ct in ['unix', 'mingw32']:
+            opts.append('-DVERSION_INFO="%s"' % self.distribution.get_version())
+            cxx_opts.append(cpp_flag(self.compiler))
+            if has_flag(self.compiler, '-fvisibility=hidden'):
+                opts.append('-fvisibility=hidden')
+            if sys.platform == 'darwin':
+                if has_flag(self.compiler, '-stdlib=libc++'):
+                    opts.append('-stdlib=libc++')
+                if has_flag(self.compiler, '-mmacosx-version-min=10.7'):
+                    opts.append('-mmacosx-version-min=10.7')
+        elif ct == 'msvc':
+            opts.append('/DVERSION_INFO=\\"%s\\"' % self.distribution.get_version())
+        for ext in self.extensions:
+            ext.extra_compile_args = ext.extra_compile_args + opts
+
+        prev__compile = self.compiler._compile
+
+        # -std=... used on C files causes an error on Apple LLVM
+        # https://gitlab.tiker.net/inducer/pymetis/-/jobs/102421
+        def _compile(obj, src, ext, cc_args, extra_postargs, pp_opts):
+            if ext == ".cpp":
+                cc_args = cc_args + cxx_opts
+
+            return prev__compile(obj, src, ext, cc_args, extra_postargs, pp_opts)
+
+        self.compiler._compile = _compile
+
+        try:
+            NumpyBuildExtCommand.build_extensions(self)
+        finally:
+            self.compiler._compile = prev__compile
+
+# }}}
+
+# }}}
+
+# vim: foldmethod=marker
