@@ -1,6 +1,4 @@
 # prepared invocations and structures -----------------------------------------
-from __future__ import print_function
-from __future__ import absolute_import
 import pycuda.driver as cuda
 import pycuda.autoinit
 import numpy
