@@ -1,13 +1,13 @@
-from __future__ import absolute_import
-from __future__ import print_function
 from pytools import memoize
+
 # don't import pycuda.driver here--you'll create an import loop
+import os
+
 import sys
 from tempfile import mkstemp
 from os import unlink
 
 from pytools.prefork import call_capture_output
-from six.moves import map
 
 
 @memoize
@@ -17,6 +17,7 @@ def get_nvcc_version(nvcc):
 
     if result != 0 or not stdout:
         from warnings import warn
+
         warn("NVCC version could not be determined.")
         stdout = "nvcc unknown version"
 
@@ -26,40 +27,48 @@ def get_nvcc_version(nvcc):
 def _new_md5():
     try:
         import hashlib
+
         return hashlib.md5()
     except ImportError:
         # for Python << 2.5
         import md5
+
         return md5.new()
 
 
 def preprocess_source(source, options, nvcc):
-    handle, source_path = mkstemp(suffix='.cu')
+    handle, source_path = mkstemp(suffix=".cu")
 
-    outf = open(source_path, 'w')
+    outf = open(source_path, "w")
     outf.write(source)
     outf.close()
     os.close(handle)
 
-    cmdline = [nvcc, '--preprocess'] + options + [source_path]
-    if 'win32' in sys.platform:
-        cmdline.extend(['--compiler-options', '-EP'])
+    cmdline = [nvcc, "--preprocess"] + options + [source_path]
+    if "win32" in sys.platform:
+        cmdline.extend(["--compiler-options", "-EP"])
     else:
-        cmdline.extend(['--compiler-options', '-P'])
+        cmdline.extend(["--compiler-options", "-P"])
 
     result, stdout, stderr = call_capture_output(cmdline, error_on_nonzero=False)
 
     if result != 0:
         from pycuda.driver import CompileError
-        raise CompileError("nvcc preprocessing of %s failed" % source_path,
-                           cmdline, stderr=stderr)
+
+        raise CompileError(
+            "nvcc preprocessing of %s failed" % source_path, cmdline, stderr=stderr
+        )
 
     # sanity check
-    if len(stdout) < 0.5*len(source):
+    if len(stdout) < 0.5 * len(source):
         from pycuda.driver import CompileError
-        raise CompileError("nvcc preprocessing of %s failed with ridiculously "
-                "small code output - likely unsupported compiler." % source_path,
-                cmdline, stderr=stderr.decode("utf-8", "replace"))
+
+        raise CompileError(
+            "nvcc preprocessing of %s failed with ridiculously "
+            "small code output - likely unsupported compiler." % source_path,
+            cmdline,
+            stderr=stderr.decode("utf-8", "replace"),
+        )
 
     unlink(source_path)
 
@@ -74,7 +83,7 @@ def compile_plain(source, options, keep, nvcc, cache_dir, target="cubin"):
     if cache_dir:
         checksum = _new_md5()
 
-        if '#include' in source:
+        if "#include" in source:
             checksum.update(preprocess_source(source, options, nvcc).encode("utf-8"))
         else:
             checksum.update(source.encode("utf-8"))
@@ -83,6 +92,7 @@ def compile_plain(source, options, keep, nvcc, cache_dir, target="cubin"):
             checksum.update(option.encode("utf-8"))
         checksum.update(get_nvcc_version(nvcc).encode("utf-8"))
         from pycuda.characterize import platform_bits
+
         checksum.update(str(platform_bits()).encode("utf-8"))
 
         cache_file = checksum.hexdigest()
@@ -99,6 +109,7 @@ def compile_plain(source, options, keep, nvcc, cache_dir, target="cubin"):
             pass
 
     from tempfile import mkdtemp
+
     file_dir = mkdtemp()
     file_root = "kernel"
 
@@ -116,12 +127,13 @@ def compile_plain(source, options, keep, nvcc, cache_dir, target="cubin"):
         print("*** compiler output in %s" % file_dir)
 
     cmdline = [nvcc, "--" + target] + options + [cu_file_name]
-    result, stdout, stderr = call_capture_output(cmdline,
-            cwd=file_dir, error_on_nonzero=False)
+    result, stdout, stderr = call_capture_output(
+        cmdline, cwd=file_dir, error_on_nonzero=False
+    )
 
     try:
         result_f = open(join(file_dir, file_root + "." + target), "rb")
-    except IOError:
+    except OSError:
         no_output = True
     else:
         no_output = False
@@ -129,22 +141,35 @@ def compile_plain(source, options, keep, nvcc, cache_dir, target="cubin"):
     if result != 0 or (no_output and (stdout or stderr)):
         if result == 0:
             from warnings import warn
-            warn("PyCUDA: nvcc exited with status 0, but appears to have "
-                    "encountered an error")
+
+            warn(
+                "PyCUDA: nvcc exited with status 0, but appears to have "
+                "encountered an error"
+            )
         from pycuda.driver import CompileError
-        raise CompileError("nvcc compilation of %s failed" % cu_file_path,
-                cmdline, stdout=stdout.decode("utf-8", "replace"),
-                stderr=stderr.decode("utf-8", "replace"))
+
+        raise CompileError(
+            "nvcc compilation of %s failed" % cu_file_path,
+            cmdline,
+            stdout=stdout.decode("utf-8", "replace"),
+            stderr=stderr.decode("utf-8", "replace"),
+        )
 
     if stdout or stderr:
-        lcase_err_text = (stdout+stderr).decode("utf-8", "replace").lower()
+        lcase_err_text = (stdout + stderr).decode("utf-8", "replace").lower()
         from warnings import warn
+
         if "demoted" in lcase_err_text or "demoting" in lcase_err_text:
-            warn("nvcc said it demoted types in source code it "
+            warn(
+                "nvcc said it demoted types in source code it "
                 "compiled--this is likely not what you want.",
-                stacklevel=4)
-        warn("The CUDA compiler succeeded, but said the following:\n"
-                + (stdout+stderr).decode("utf-8", "replace"), stacklevel=4)
+                stacklevel=4,
+            )
+        warn(
+            "The CUDA compiler succeeded, but said the following:\n"
+            + (stdout + stderr).decode("utf-8", "replace"),
+            stacklevel=4,
+        )
 
     result_data = result_f.read()
     result_f.close()
@@ -156,6 +181,7 @@ def compile_plain(source, options, keep, nvcc, cache_dir, target="cubin"):
 
     if not keep:
         from os import listdir, unlink, rmdir
+
         for name in listdir(file_dir):
             unlink(join(file_dir, name))
         rmdir(file_dir)
@@ -169,6 +195,7 @@ def _get_per_user_string():
     except ImportError:
         checksum = _new_md5()
         from os import environ
+
         checksum.update(environ["USERNAME"].encode("utf-8"))
         return checksum.hexdigest()
     else:
@@ -177,19 +204,29 @@ def _get_per_user_string():
 
 def _find_pycuda_include_path():
     from pkg_resources import Requirement, resource_filename
+
     return resource_filename(Requirement.parse("pycuda"), "pycuda/cuda")
 
 
-import os
 DEFAULT_NVCC_FLAGS = [
-        _flag.strip() for _flag in
-        os.environ.get("PYCUDA_DEFAULT_NVCC_FLAGS", "").split()
-        if _flag.strip()]
+    _flag.strip()
+    for _flag in os.environ.get("PYCUDA_DEFAULT_NVCC_FLAGS", "").split()
+    if _flag.strip()
+]
 
 
-def compile(source, nvcc="nvcc", options=None, keep=False,
-        no_extern_c=False, arch=None, code=None, cache_dir=None,
-        include_dirs=[], target="cubin"):
+def compile(
+    source,
+    nvcc="nvcc",
+    options=None,
+    keep=False,
+    no_extern_c=False,
+    arch=None,
+    code=None,
+    cache_dir=None,
+    include_dirs=[],
+    target="cubin",
+):
 
     assert target in ["cubin", "ptx", "fatbin"]
 
@@ -202,13 +239,16 @@ def compile(source, nvcc="nvcc", options=None, keep=False,
     options = options[:]
     if arch is None:
         from pycuda.driver import Error
+
         try:
             from pycuda.driver import Context
+
             arch = "sm_%d%d" % Context.get_device().compute_capability()
         except Error:
             pass
 
     from pycuda.driver import CUDA_DEBUGGING
+
     if CUDA_DEBUGGING:
         cache_dir = False
         keep = True
@@ -222,14 +262,18 @@ def compile(source, nvcc="nvcc", options=None, keep=False,
 
     if cache_dir is None:
         import appdirs
-        cache_dir = os.path.join(appdirs.user_cache_dir("pycuda", "pycuda"),
-                "compiler-cache-v1")
+
+        cache_dir = os.path.join(
+            appdirs.user_cache_dir("pycuda", "pycuda"), "compiler-cache-v1"
+        )
 
         from os import makedirs
+
         try:
             makedirs(cache_dir)
         except OSError as e:
             from errno import EEXIST
+
             if e.errno != EEXIST:
                 raise
 
@@ -239,32 +283,36 @@ def compile(source, nvcc="nvcc", options=None, keep=False,
     if code is not None:
         options.extend(["-code", code])
 
-    if 'darwin' in sys.platform and sys.maxsize == 9223372036854775807:
-        options.append('-m64')
-    elif 'win32' in sys.platform and sys.maxsize == 9223372036854775807:
-        options.append('-m64')
-    elif 'win32' in sys.platform and sys.maxsize == 2147483647:
-        options.append('-m32')
+    if "darwin" in sys.platform and sys.maxsize == 9223372036854775807:
+        options.append("-m64")
+    elif "win32" in sys.platform and sys.maxsize == 9223372036854775807:
+        options.append("-m64")
+    elif "win32" in sys.platform and sys.maxsize == 2147483647:
+        options.append("-m32")
 
     include_dirs = include_dirs + [_find_pycuda_include_path()]
 
     for i in include_dirs:
-        options.append("-I"+i)
+        options.append("-I" + i)
 
     return compile_plain(source, options, keep, nvcc, cache_dir, target)
 
 
-class CudaModule(object):
+class CudaModule:
     def _check_arch(self, arch):
         if arch is None:
             return
         try:
             from pycuda.driver import Context
+
             capability = Context.get_device().compute_capability()
             if tuple(map(int, tuple(arch.split("_")[1]))) > capability:
                 from warnings import warn
-                warn("trying to compile for a compute capability "
-                        "higher than selected GPU")
+
+                warn(
+                    "trying to compile for a compute capability "
+                    "higher than selected GPU"
+                )
         except Exception:
             pass
 
@@ -277,20 +325,41 @@ class CudaModule(object):
     def get_function(self, name):
         return self.module.get_function(name)
 
+
 class SourceModule(CudaModule):
-    '''
+    """
     Creates a Module from a single .cu source object linked against the
     static CUDA runtime.
-    '''
-    def __init__(self, source, nvcc="nvcc", options=None, keep=False,
-            no_extern_c=False, arch=None, code=None, cache_dir=None,
-            include_dirs=[]):
+    """
+
+    def __init__(
+        self,
+        source,
+        nvcc="nvcc",
+        options=None,
+        keep=False,
+        no_extern_c=False,
+        arch=None,
+        code=None,
+        cache_dir=None,
+        include_dirs=[],
+    ):
         self._check_arch(arch)
 
-        cubin = compile(source, nvcc, options, keep, no_extern_c,
-                arch, code, cache_dir, include_dirs)
+        cubin = compile(
+            source,
+            nvcc,
+            options,
+            keep,
+            no_extern_c,
+            arch,
+            code,
+            cache_dir,
+            include_dirs,
+        )
 
         from pycuda.driver import module_from_buffer
+
         self.module = module_from_buffer(cubin)
 
         self._bind_module()
@@ -318,21 +387,36 @@ def _find_nvcc_on_path():
 
 
 class DynamicModule(CudaModule):
-    '''
+    """
     Creates a Module from multiple .cu source, library file and/or data
     objects linked against the static or dynamic CUDA runtime.
-    '''
-    def __init__(self, nvcc='nvcc', link_options=None, keep=False,
-            no_extern_c=False, arch=None, code=None, cache_dir=None,
-            include_dirs=[],  message_handler=None, log_verbose=False,
-            cuda_libdir=None):
+    """
+
+    def __init__(
+        self,
+        nvcc="nvcc",
+        link_options=None,
+        keep=False,
+        no_extern_c=False,
+        arch=None,
+        code=None,
+        cache_dir=None,
+        include_dirs=[],
+        message_handler=None,
+        log_verbose=False,
+        cuda_libdir=None,
+    ):
         from pycuda.driver import Context
+
         compute_capability = Context.get_device().compute_capability()
-        if compute_capability < (3,5):
-            raise Exception('Minimum compute capability for dynamic parallelism is 3.5 (found: %u.%u)!' %
-                (compute_capability[0], compute_capability[1]))
+        if compute_capability < (3, 5):
+            raise Exception(
+                "Minimum compute capability for dynamic parallelism is 3.5 (found: %u.%u)!"
+                % (compute_capability[0], compute_capability[1])
+            )
         else:
             from pycuda.driver import Linker
+
             self.linker = Linker(message_handler, link_options, log_verbose)
         self._check_arch(arch)
         self.nvcc = nvcc
@@ -347,7 +431,7 @@ class DynamicModule(CudaModule):
         self.module = None
 
     def _locate_cuda_libdir(self):
-        '''
+        """
         Locate the "standard" CUDA SDK library directory in the local
         file system. Supports 64-Bit Windows, Linux and Mac OS X.
         In case the caller supplied cuda_libdir in the constructor
@@ -371,52 +455,68 @@ class DynamicModule(CudaModule):
           https://cmake.org/cmake/help/v3.0/module/FindCUDA.html
           https://github.com/Kitware/CMake/blob/master/Modules/FindCUDA.cmake
         - Verify all Linux code paths somehow
-        '''
+        """
         from os.path import isfile, join
         from platform import system as platform_system
+
         system = platform_system()
         libdir, libptn = None, None
-        if system == 'Windows':
+        if system == "Windows":
             if self.cuda_libdir is not None:
                 libdir = self.cuda_libdir
-            elif 'CUDA_PATH' in os.environ and isfile(join(os.environ['CUDA_PATH'], 'lib\\x64\\cudadevrt.lib')):
-                libdir = join(os.environ['CUDA_PATH'], 'lib\\x64')
-            libptn = '%s.lib'
-        elif system in ['Linux', 'Darwin']:
+            elif "CUDA_PATH" in os.environ and isfile(
+                join(os.environ["CUDA_PATH"], "lib\\x64\\cudadevrt.lib")
+            ):
+                libdir = join(os.environ["CUDA_PATH"], "lib\\x64")
+            libptn = "%s.lib"
+        elif system in ["Linux", "Darwin"]:
             if self.cuda_libdir is not None:
                 libdir = self.cuda_libdir
-            elif 'CUDA_ROOT' in os.environ and isfile(join(os.environ['CUDA_ROOT'], 'lib64/libcudadevrt.a')):
-                libdir = join(os.environ['CUDA_ROOT'], 'lib64')
-            elif 'LD_LIBRARY_PATH' in os.environ:
-                for ld_path in os.environ['LD_LIBRARY_PATH'].split(':'):
-                    if isfile(join(ld_path, 'libcudadevrt.a')):
+            elif "CUDA_ROOT" in os.environ and isfile(
+                join(os.environ["CUDA_ROOT"], "lib64/libcudadevrt.a")
+            ):
+                libdir = join(os.environ["CUDA_ROOT"], "lib64")
+            elif "LD_LIBRARY_PATH" in os.environ:
+                for ld_path in os.environ["LD_LIBRARY_PATH"].split(":"):
+                    if isfile(join(ld_path, "libcudadevrt.a")):
                         libdir = ld_path
                         break
 
-            if libdir is None and isfile('/usr/lib/x86_64-linux-gnu/libcudadevrt.a'):
-                libdir = '/usr/lib/x86_64-linux-gnu'
+            if libdir is None and isfile("/usr/lib/x86_64-linux-gnu/libcudadevrt.a"):
+                libdir = "/usr/lib/x86_64-linux-gnu"
 
             if libdir is None:
                 nvcc_path = _find_nvcc_on_path()
                 if nvcc_path is not None:
                     libdir = join(os.path.dirname(nvcc_path), "..", "lib64")
 
-            libptn = 'lib%s.a'
+            libptn = "lib%s.a"
         if libdir is None:
-            raise RuntimeError('Unable to locate the CUDA SDK installation '
-                'directory, set CUDA library path manually')
+            raise RuntimeError(
+                "Unable to locate the CUDA SDK installation "
+                "directory, set CUDA library path manually"
+            )
         return libdir, libptn
 
-    def add_source(self, source, nvcc_options=None, name='kernel.ptx'):
-        ptx = compile(source, nvcc=self.nvcc, options=nvcc_options,
-            keep=self.keep, no_extern_c=self.no_extern_c, arch=self.arch,
-            code=self.code, cache_dir=self.cache_dir,
-            include_dirs=self.include_dirs, target="ptx")
+    def add_source(self, source, nvcc_options=None, name="kernel.ptx"):
+        ptx = compile(
+            source,
+            nvcc=self.nvcc,
+            options=nvcc_options,
+            keep=self.keep,
+            no_extern_c=self.no_extern_c,
+            arch=self.arch,
+            code=self.code,
+            cache_dir=self.cache_dir,
+            include_dirs=self.include_dirs,
+            target="ptx",
+        )
         from pycuda.driver import jit_input_type
+
         self.linker.add_data(ptx, jit_input_type.PTX, name)
         return self
 
-    def add_data(self, data, input_type, name='unknown'):
+    def add_data(self, data, input_type, name="unknown"):
         self.linker.add_data(data, input_type, name)
         return self
 
@@ -428,10 +528,12 @@ class DynamicModule(CudaModule):
         if self.libdir is None:
             self.libdir, self.libptn = self._locate_cuda_libdir()
         from os.path import isfile, join
+
         libpath = join(self.libdir, self.libptn % libname)
         if not isfile(libpath):
             raise OSError('CUDA SDK library file "%s" not found' % libpath)
         from pycuda.driver import jit_input_type
+
         self.linker.add_file(libpath, jit_input_type.LIBRARY)
         return self
 
@@ -443,28 +545,46 @@ class DynamicModule(CudaModule):
 
 
 class DynamicSourceModule(DynamicModule):
-    '''
+    """
     Creates a Module from a single .cu source object linked against the
     dynamic CUDA runtime.
     - compiler generates PTX relocatable device code (rdc) from source that
       can be linked with other relocatable device code
     - source is linked against the CUDA device runtime library cudadevrt
     - library cudadevrt is statically linked into the generated Module
-    '''
-    def __init__(self, source, nvcc="nvcc", options=None, keep=False,
-            no_extern_c=False, arch=None, code=None, cache_dir=None,
-            include_dirs=[], cuda_libdir=None):
-        super(DynamicSourceModule, self).__init__(nvcc=nvcc,
-            link_options=None, keep=keep, no_extern_c=no_extern_c,
-            arch=arch, code=code, cache_dir=cache_dir,
-            include_dirs=include_dirs, cuda_libdir=cuda_libdir)
+    """
+
+    def __init__(
+        self,
+        source,
+        nvcc="nvcc",
+        options=None,
+        keep=False,
+        no_extern_c=False,
+        arch=None,
+        code=None,
+        cache_dir=None,
+        include_dirs=[],
+        cuda_libdir=None,
+    ):
+        super().__init__(
+            nvcc=nvcc,
+            link_options=None,
+            keep=keep,
+            no_extern_c=no_extern_c,
+            arch=arch,
+            code=code,
+            cache_dir=cache_dir,
+            include_dirs=include_dirs,
+            cuda_libdir=cuda_libdir,
+        )
         if options is None:
             options = DEFAULT_NVCC_FLAGS
         options = options[:]
-        if '-rdc=true' not in options:
-            options.append('-rdc=true')
-        if '-lcudadevrt' not in options:
-            options.append('-lcudadevrt')
+        if "-rdc=true" not in options:
+            options.append("-rdc=true")
+        if "-lcudadevrt" not in options:
+            options.append("-lcudadevrt")
         self.add_source(source, nvcc_options=options)
-        self.add_stdlib('cudadevrt')
+        self.add_stdlib("cudadevrt")
         self.link()
